/*###############################################################################
# Linux Management Providers (LMP), EthernetPort provider package
# Copyright (C) 2008 Guillaume BOTTEX, ETRI <guillaumebottex@etri.re.kr, guillaumebottex@gmail.com>
# 
# This program is being developed under the "OpenDRIM" project.
# The "OpenDRIM" project web page: http://opendrim.sourceforge.net
# The "OpenDRIM" project mailing list: opendrim@googlegroups.com
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2
# of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#################################################################################

#################################################################################
# To contributors, please leave your contact information in this section
# AND comment your changes in the source code.
# 
# Modified by <Author(s)>, <Affiliation>, <Year>
###############################################################################*/

#include "cmpiOpenDRIM_EthernetPortDeviceSAPImplementation.h"
#include "OpenDRIM_EthernetPortDeviceSAPImplementationAccess.h"

static const CMPIBroker * _broker;

#ifdef CMPI_VER_100
#define EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderSetInstance EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderModifyInstance
#endif

int EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_init(const CMPIBroker* broker);

int EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_finalize();

/* ---------------------------------------------------------------------------*/
/*                      Instance Provider Interface                           */
/* ---------------------------------------------------------------------------*/

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderCleanup(CMPIInstanceMI * mi, const CMPIContext* ctx, CMPIBoolean terminate) {
	_E_;
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_finalize();
	CMPIStatus rc;
	CMSetStatus(&rc, (CMPIrc) errorCode);
	_L_;
	return rc;
}

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderEnumInstanceNames(CMPIInstanceMI* mi, const CMPIContext* ctx, const CMPIResult* rslt, const CMPIObjectPath* ref) {
	_E_;
	CMPIStatus rc = {CMPI_RC_OK, NULL};
	vector<OpenDRIM_EthernetPortDeviceSAPImplementation> instances;
	string errorMessage;
	
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_retrieve(_broker, ctx, instances, NULL, errorMessage, "ein");
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	for (size_t i=0; i < instances.size(); i++) {
		CMPIObjectPath* op = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_toCMPIObjectPath(_broker, instances[i]);
		CMReturnObjectPath(rslt, op);
	}
	CMReturnDone(rslt);
	_L_;
	return rc;
}

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderEnumInstances(CMPIInstanceMI* mi, const CMPIContext* ctx, const CMPIResult* rslt, const CMPIObjectPath* ref, const char** properties) {
	_E_;
	CMPIStatus rc = {CMPI_RC_OK, NULL};
	vector<OpenDRIM_EthernetPortDeviceSAPImplementation> instances;
	string errorMessage;
	
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_retrieve(_broker, ctx, instances, properties, errorMessage, "ei");
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	for (size_t i=0; i < instances.size(); i++) {
		CMPIInstance* ci = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_toCMPIInstance(_broker, instances[i]);
		CMReturnInstance(rslt, ci);
	}
	CMReturnDone(rslt);
	_L_;
	return rc;
}

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderGetInstance(CMPIInstanceMI* mi, const CMPIContext* ctx, const CMPIResult* rslt, const CMPIObjectPath* cop, const char** properties) {
	_E_;
	CMPIStatus rc = {CMPI_RC_OK, NULL};
	OpenDRIM_EthernetPortDeviceSAPImplementation instance;
	EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_toCPP(_broker, cop, instance);
	string errorMessage;
	
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_getInstance(_broker, ctx, instance, properties, errorMessage);
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	CMPIInstance* ci = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_toCMPIInstance(_broker, instance);
	CMReturnInstance(rslt, ci);
	CMReturnDone(rslt);
	_L_;
	return rc;
}

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderSetInstance(CMPIInstanceMI* mi, const CMPIContext* ctx, const CMPIResult* rslt, const CMPIObjectPath* cop, const CMPIInstance* ci, const char** properties) {
	_E_;
	CMPIStatus rc = {CMPI_RC_OK, NULL};
	OpenDRIM_EthernetPortDeviceSAPImplementation newInstance, oldInstance;
	EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_toCPP(_broker, ci, newInstance);
	EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_toCPP(_broker, cop, oldInstance);
	string errorMessage;
	
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_getInstance(_broker, ctx, oldInstance, properties, errorMessage);
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_setInstance(_broker, ctx, newInstance, oldInstance, properties, errorMessage);
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	CMReturnDone(rslt);
	_L_;
	return rc;
}

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderCreateInstance(CMPIInstanceMI* mi, const CMPIContext* ctx, const CMPIResult* rslt, const CMPIObjectPath* cop, const CMPIInstance* ci) {
	_E_;
	CMPIStatus rc = {CMPI_RC_OK, NULL};
	OpenDRIM_EthernetPortDeviceSAPImplementation instance;
	EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_toCPP(_broker, ci, instance);
	string errorMessage;
	
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_getInstance(_broker, ctx, instance, NULL, errorMessage);
	if (errorCode != NOT_FOUND) {
		if (errorCode == OK)
			errorCode = ALREADY_EXISTS;
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_createInstance(_broker, ctx, instance, errorMessage);
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_getInstance(_broker, ctx, instance, NULL, errorMessage);
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	CMPIObjectPath* _cop = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_toCMPIObjectPath(_broker, instance);
	CMReturnObjectPath(rslt, _cop);
	CMReturnDone(rslt);
	_L_;
	return rc;
}

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderDeleteInstance(CMPIInstanceMI* mi, const CMPIContext* ctx, const CMPIResult* rslt, const CMPIObjectPath* cop) {
	_E_;
	CMPIStatus rc = {CMPI_RC_OK, NULL};
	OpenDRIM_EthernetPortDeviceSAPImplementation instance;
	EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_toCPP(_broker, cop, instance);
	string errorMessage;
	
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_getInstance(_broker, ctx, instance, NULL, errorMessage);
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_deleteInstance(_broker, ctx, instance, errorMessage);
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	CMReturnDone(rslt);
	_L_;
	return rc;
}

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderExecQuery(CMPIInstanceMI* mi, const CMPIContext* ctx, const CMPIResult* rslt, const CMPIObjectPath* ref, const char* lang, const char* query) {
	_E_;
	CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

/* ---------------------------------------------------------------------------*/
/*                   End of Instance Provider Interface                       */
/* ---------------------------------------------------------------------------*/

/* ---------------------------------------------------------------------------*/
/*                     Association Provider Interface                         */
/* ---------------------------------------------------------------------------*/

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderAssociationCleanup(CMPIAssociationMI * mi, const CMPIContext* ctx, CMPIBoolean terminate) {
	_E_;
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_finalize();
	CMPIStatus rc;
	CMSetStatus(&rc, (CMPIrc) errorCode);
	_L_;
	return rc;
}

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderAssociatorNames(CMPIAssociationMI* mi, const CMPIContext* ctx, const CMPIResult* rslt, const CMPIObjectPath* cop, const char* assocClass, const char* resultClass, const char* role, const char* resultRole) {
	_E_;
	CMPIStatus rc = {CMPI_RC_OK, NULL};
	
	string errorMessage;
	bool leftToRight;
	vector<Instance> associatedInstances;
	Instance known;
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_associator(_broker, ctx, cop, known, assocClass, resultClass, role, resultRole, NULL, associatedInstances, leftToRight, errorMessage, "an");
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	
	for (size_t i=0; i < associatedInstances.size(); i++)
		CMReturnObjectPath(rslt, associatedInstances[i].getObjectpath().getHdl());
	CMReturnDone(rslt);
	_L_;
	return rc;
}

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderAssociators(CMPIAssociationMI* mi, const CMPIContext* ctx, const CMPIResult* rslt, const CMPIObjectPath* cop, const char* assocClass, const char* resultClass, const char* role, const char* resultRole, const char** properties) {
	_E_;
	CMPIStatus rc = {CMPI_RC_OK, NULL};
	
	string errorMessage;
	bool leftToRight;
	vector<Instance> associatedInstances;
	Instance known;
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_associator(_broker, ctx, cop, known, assocClass, resultClass, role, resultRole, properties, associatedInstances, leftToRight, errorMessage, "a");
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	
	for (size_t i=0; i < associatedInstances.size(); i++)
		CMReturnInstance(rslt, associatedInstances[i].getHdl());
	CMReturnDone(rslt);
	_L_;
	return rc;
}

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderReferenceNames(CMPIAssociationMI* mi, const CMPIContext* ctx, const CMPIResult* rslt, const CMPIObjectPath* cop, const char* assocClass, const char* role) {
	_E_;
	CMPIStatus rc = {CMPI_RC_OK, NULL};
	
	string errorMessage;
	bool leftToRight;
	vector<Instance> associatedInstances;
	Instance known;
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_associator(_broker, ctx, cop, known, assocClass, NULL, role, NULL, NULL, associatedInstances, leftToRight, errorMessage, "an");
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	
	vector<OpenDRIM_EthernetPortDeviceSAPImplementation> associationInstances;
	if (leftToRight)
		errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_referenceAntecedentToDependent(_broker, ctx, known, associatedInstances, associationInstances, NULL, errorMessage, "rn");
	else
		errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_referenceDependentToAntecedent(_broker, ctx, known, associatedInstances, associationInstances, NULL, errorMessage, "rn");
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	
	for (size_t i=0; i < associationInstances.size(); i++) {
		CMPIObjectPath* op = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_toCMPIObjectPath(_broker, associationInstances[i]);
		CMReturnObjectPath(rslt, op);
	}
	_L_;
	return rc;
}

CMPIStatus EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProviderReferences(CMPIAssociationMI* mi, const CMPIContext* ctx, const CMPIResult* rslt, const CMPIObjectPath* cop, const char* assocClass, const char* role, const char** properties) {
	_E_;
	CMPIStatus rc = {CMPI_RC_OK, NULL};
	
	string errorMessage;
	bool leftToRight;
	vector<Instance> associatedInstances;
	Instance known;
	int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_associator(_broker, ctx, cop, known, assocClass, NULL, role, NULL, NULL, associatedInstances, leftToRight, errorMessage, "an");
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	
	vector<OpenDRIM_EthernetPortDeviceSAPImplementation> associationInstances;
	if (leftToRight)
		errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_referenceAntecedentToDependent(_broker, ctx, known, associatedInstances, associationInstances, properties, errorMessage, "r");
	else
		errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_referenceDependentToAntecedent(_broker, ctx, known, associatedInstances, associationInstances, properties, errorMessage, "r");
	if (errorCode != OK) {
		errorMessage = (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
		CMSetStatusWithChars(_broker, &rc, (CMPIrc) errorCode, errorMessage.c_str());
		return rc;
	}
	
	for (size_t i=0; i < associationInstances.size(); i++) {
		CMPIInstance* inst = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_toCMPIInstance(_broker, associationInstances[i]);
		CMReturnInstance(rslt, inst);
	}
	_L_;
	return rc;
}

/* ---------------------------------------------------------------------------*/
/*                  End Of Association Provider Interface                     */
/* ---------------------------------------------------------------------------*/

/* ---------------------------------------------------------------------------*/
/*                              Provider Factory                              */
/* ---------------------------------------------------------------------------*/

int EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_init(const CMPIBroker* broker) {
	_E_;
	_broker = broker;
	static bool initialized = false;
	if (!initialized) {
		string errorMessage;
		int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_load(broker, errorMessage);
		if (errorCode != OK) {
			errorMessage += "EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_init FAILED: " + (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
			string cmd = "/bin/echo \"" + errorMessage + "\" >> cmpi_prov_debug.txt";
			system(cmd.c_str());
			return -1;
		}
		initialized = true;
	}
	_L_;
	return OK;
}

int EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_finalize() {
	_E_;
	static bool finalized = false;
	if (!finalized) {
		string errorMessage;
		int errorCode = EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_unload(errorMessage);
		if (errorCode != OK) {
			errorMessage += "EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_unload FAILED: " + (string) OpenDRIM_EthernetPortDeviceSAPImplementation_classnames[0] + ": " + errorMessage;
			string cmd = "/bin/echo \"" + errorMessage + "\" >> cmpi_prov_debug.txt";
			system(cmd.c_str());
			return errorCode;
		}
		finalized = true;
	}
	_L_;
	return OK;
}

#define EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_INIT if (EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_init(_broker) < 0) return NULL

CMInstanceMIStub(EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProvider, EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProvider, _broker, EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_INIT);
CMAssociationMIStub(EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProvider, EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementationProvider, _broker, EthernetPort_OpenDRIM_EthernetPortDeviceSAPImplementation_INIT);

/* ---------------------------------------------------------------------------*/
/*                          End of Provider Factory                           */
/* ---------------------------------------------------------------------------*/
